@@ -1,12 +1,8 @@
 #ifndef _LINUX_FLIST_H
 #define _LINUX_FLIST_H
 
-#undef offsetof
-#ifdef __compiler_offsetof
-#define offsetof(TYPE,MEMBER) __compiler_offsetof(TYPE,MEMBER)
-#else
-#define offsetof(TYPE, MEMBER) ((size_t) &((TYPE *)0)->MEMBER)
-#endif
+#include <stdlib.h>
+#include <stddef.h>
 
 #define container_of(ptr, type, member) ({			\
 	const typeof( ((type *)0)->member ) *__mptr = (ptr);	\
@@ -117,6 +113,52 @@ static inline int flist_empty(const struct flist_head *head)
 	return head->next == head;
 }
 
+static inline void __flist_splice(const struct flist_head *list,
+				  struct flist_head *prev,
+				  struct flist_head *next)
+{
+	struct flist_head *first = list->next;
+	struct flist_head *last = list->prev;
+
+	first->prev = prev;
+	prev->next = first;
+
+	last->next = next;
+	next->prev = last;
+}
+
+static inline void flist_splice(const struct flist_head *list,
+				struct flist_head *head)
+{
+	if (!flist_empty(list))
+		__flist_splice(list, head, head->next);
+}
+
+static inline void flist_splice_tail(struct flist_head *list,
+				     struct flist_head *head)
+{
+	if (!flist_empty(list))
+		__flist_splice(list, head->prev, head);
+}
+
+static inline void flist_splice_tail_init(struct flist_head *list,
+					  struct flist_head *head)
+{
+	if (!flist_empty(list)) {
+		__flist_splice(list, head->prev, head);
+		INIT_FLIST_HEAD(list);
+	}
+}
+
+static inline void flist_splice_init(struct flist_head *list,
+				    struct flist_head *head)
+{
+	if (!flist_empty(list)) {
+		__flist_splice(list, head, head->next);
+		INIT_FLIST_HEAD(list);
+	}
+}
+
 /**
  * flist_entry - get the struct for this entry
  * @ptr:	the &struct flist_head pointer.
@@ -126,6 +168,12 @@ static inline int flist_empty(const struct flist_head *head)
 #define flist_entry(ptr, type, member) \
 	container_of(ptr, type, member)
 
+#define flist_first_entry(ptr, type, member) \
+	flist_entry((ptr)->next, type, member)
+
+#define flist_last_entry(ptr, type, member) \
+	flist_entry((ptr)->prev, type, member)
+
 /**
  * flist_for_each	-	iterate over a list
  * @pos:	the &struct flist_head to use as a loop counter.
@@ -144,4 +192,7 @@ static inline int flist_empty(const struct flist_head *head)
 	for (pos = (head)->next, n = pos->next; pos != (head); \
 		pos = n, n = pos->next)
 
+extern void flist_sort(void *priv, struct flist_head *head,
+	int (*cmp)(void *priv, struct flist_head *a, struct flist_head *b));
+
 #endif
